@@ -1,5 +1,4 @@
 import copy
-import queue
 from typing import Callable
 
 import numpy as np
@@ -52,7 +51,7 @@ class DataPipeline:
         self._data = None
         self._use_gpu = False
         self._end_function = None
-        self._operations = queue.Queue()
+        self._operations = []
         self._is_run = False
         self._cuda_stream = None
 
@@ -104,7 +103,7 @@ class DataPipeline:
         if self._is_run:
             raise PipelineAlreadyRunException()
 
-        self._operations.put(operation)
+        self._operations.append(operation)
         return self
 
     def run(self, use_gpu: bool) -> 'DataPipeline':
@@ -136,15 +135,19 @@ class DataPipeline:
         # and restore data from GPU (after executing the pipeline) is mandatory
         if self._use_gpu:
             self._cuda_stream = engine.cuda.Stream(null=False, non_blocking=True)
-            self._cuda_stream.use()
-            self._data = engine.asarray(self._data)
             # Transfer data from device at the end of the pipeline
-            self._operations.put(lambda data, engine: (engine.asnumpy(data, stream=self._cuda_stream), engine))
+            self._operations.append(lambda data, engine: (engine.asnumpy(data, stream=self._cuda_stream), engine))
 
         ret = self._data
-        for operation in self._operations.queue:
 
-            ret, engine = operation(ret, engine)
+        if self._use_gpu:
+            with self._cuda_stream:
+                ret = engine.asarray(ret)
+                for operation in self._operations:
+                    ret, engine = operation(ret, engine)
+        else:
+            for operation in self._operations:
+                ret, engine = operation(ret, engine)
 
         self._data = ret
 
@@ -168,12 +171,12 @@ class DataPipeline:
 
         return ret
 
-    def set_operations(self, operations: queue.Queue) -> 'DataPipeline':
+    def set_operations(self, operations: list) -> 'DataPipeline':
         """
-        Replaces the operations queue with the input one.
-        :raise PipelineAlreadyRunException if the pipeline has been already run. You cannot set the operation queue of a run pipeline
-        :param operations: the queue with the operations
-        :type operations: Queue
+        Replaces the operations list with the input one.
+        :raise PipelineAlreadyRunException if the pipeline has been already run. You cannot set the operation list of a run pipeline
+        :param operations: the list with the operations
+        :type operations: List
         :return: the pipeline instance
         :rtype: DataPipeline
         """
@@ -183,15 +186,14 @@ class DataPipeline:
         self._operations = operations
         return self
 
-    def get_operations(self) -> queue.Queue:
+    def get_operations(self) -> list:
         """
-        Returns a copy of the queue containing the pipeline operations.
-        :raise PipelineAlreadyRunException if the pipeline has been already run. You cannot get the operation queue of a run pipeline
-        :return: the queue with the operations
-        :rtype: Queue
+        Returns a copy of the list containing the pipeline operations.
+        :raise PipelineAlreadyRunException if the pipeline has been already run. You cannot get the operation list of a run pipeline
+        :return: the list with the operations
+        :rtype: list
         """
         if self._is_run:
             raise PipelineAlreadyRunException()
 
-        operation = copy.copy(self._operations)
-        return operation
\ No newline at end of file
+        return self._operations
\ No newline at end of file
